from os import getenv

import docker
from flask import Flask, jsonify, render_template, request

app = Flask(__name__, static_folder="static")

client = docker.from_env()


def get_containers_info():
    containers = client.containers.list(all=True)
    container_info = []
    seen_ids = set()

    for container in containers:
        ports = container.attrs["NetworkSettings"]["Ports"]

        for name, data in container.attrs["NetworkSettings"]["Networks"].items():
            network = f"{name}: {data['IPAddress']}" if name != "host" else "host"

        container_id = container.id[:12]
        if container_id in seen_ids:
            continue  # Skip duplicate containers
        seen_ids.add(container_id)

        host_ports = []
        docker_ports = []
        for i in ports.items():
            if isinstance(i[1], list):
                docker_port, [host_binding, _] = i
            else:
                docker_port, host_binding = i
            docker_ports.append(docker_port)
            host_ports.append(
                host_binding["HostPort"] if host_binding is not None else "None"
            )

        if host_ports == []:
            host_ports = ["None"]
        if docker_ports == []:
            docker_ports = ["None"]

        info = {
            "name": container.name,
            "image": container.image.tags[0] if container.image.tags else "Unknown",
            "status": container.status,
            "docker_ports": docker_ports,
            "host_ports": host_ports,
            "created": container.attrs["Created"],
            "id": container_id,  # Shortened container ID
            "network": network,
        }
        container_info.append(info)

    return container_info


@app.route("/api/containers", methods=["GET"])
def containers():
    containers = get_containers_info()
    return jsonify(containers)


@app.route("/api/containers/<container_id>", methods=["GET"])
def get_container_status(container_id):
    container = client.containers.get(container_id)
    container.reload()
    return jsonify({"status": container.status})


@app.route("/api/containers/<container_id>/stop", methods=["POST"])
def stop_container(container_id):
    container = client.containers.get(container_id)
    container.stop()
    return jsonify({"status": "stopped"})


@app.route("/api/containers/<container_id>/restart", methods=["POST"])
def restart_container(container_id):
    container = client.containers.get(container_id)
    container.restart()
    # Wait for the container to restart and get the updated status
    container.reload()
    return jsonify({"status": container.status})


@app.route("/")
def index():
    return render_template("index.html")


@app.route("/image-port-list")
def image_port_list():
    return render_template("image-port-list.html")


def main():
    port = int(getenv("FLASK_PORT", 3000))
    app.run(debug=False, host="0.0.0.0", port=port)


if __name__ == "__main__":
    main()
