let containersData = [];
let dataTable;

async function fetchContainers() {
  try {
    const response = await fetch('/api/containers');
    containersData = await response.json();
    populateTable(containersData);
  } catch (error) {
    console.error('Error fetching containers:', error);
  }
}

function populateTable(data) {
  const tableBody = document.getElementById('containers-table-body');
  tableBody.innerHTML = '';

  data.forEach(container => {
    const row = document.createElement('tr');
    row.innerHTML = `
      <td>${container.name}</td>
      <td>${container.image}</td>
      <td>
        <span class="status-text">${container.status}</span>
        <span class="loading-icon" style="display: none;">&#x1F3C3;</span>
      </td>
      <td>${container.host_ports.join(', ')}</td>
      <td>${container.docker_ports.join(', ')}</td>
      <td>${new Date(container.created).toLocaleString()}</td>
      <td>${container.network}</td>
      <td>${container.id}</td>
      <td class="action-buttons">
        <button class="btn btn-warning btn-sm" onclick="stopContainer('${container.id}')">Stop</button>
        <button class="btn btn-success btn-sm" onclick="restartContainer('${container.id}')">Restart</button>
      </td>
    `;
    tableBody.appendChild(row);
  });

  // Initialize DataTables after populating the table
  if ($.fn.DataTable.isDataTable('#dockerTable')) {
    dataTable.clear().rows.add(data).draw();
  } else {
    dataTable = $('#dockerTable').DataTable({
      pageLength: 25,
      columnDefs: [
        {
          targets: [3, 4], // Host Port and Docker Port columns
          type: 'port-sort'
        },
        {
          targets: 6, // Network column
          type: 'network-sort'
        },
        {
          targets: 5, // Created column
          type: 'date'
        }
      ]
    });
  }
}

// Custom sorting function for ports
$.fn.dataTable.ext.type.order['port-sort-pre'] = function(data) {
  if (data) {
    const firstPort = data.split(',')[0];
    if (firstPort === 'None') {
      return Number.MAX_SAFE_INTEGER; // Move 'None' to the end of the sort
    }
    const portNumber = firstPort.split('/')[0];
    return parseInt(portNumber, 10);
  }
  return Number.MAX_SAFE_INTEGER; // Handle empty or malformed data
};

// Custom sorting function for network
$.fn.dataTable.ext.type.order['network-sort-pre'] = function(data) {
  if (data === 'host') {
    return Number.MAX_SAFE_INTEGER; // Move 'host' to the end of the sort
  }
  return data;
};
async function stopContainer(containerId) {
	try {
		const rows = dataTable.rows().nodes();
		let row;
		rows.each(function (r) {
			if (r.cells[7].textContent === containerId) {
				row = r;
			}
		});

		if (!row) {
			throw new Error('Container row not found');
		}

		const statusText = row.querySelector('.status-text');
		const loadingIcon = row.querySelector('.loading-icon');

		statusText.textContent = 'stopping';
		loadingIcon.style.display = 'inline';

		await fetch(`/api/containers/${containerId}/stop`, { method: 'POST' });

		statusText.textContent = 'stopped';
		loadingIcon.style.display = 'none';
		dataTable.row(row).invalidate().draw(false); // Refresh the specific row
	} catch (error) {
		console.error('Error stopping container:', error);
	}
}

async function restartContainer(containerId) {
  try {
    const rows = dataTable.rows().nodes();
    let row;
    rows.each(function (r) {
      if (r.cells[7].textContent === containerId) {
        row = r;
      }
    });

    if (!row) {
      throw new Error('Container row not found');
    }

    const statusText = row.querySelector('.status-text');
    const loadingIcon = row.querySelector('.loading-icon');

    statusText.textContent = 'restarting';
    loadingIcon.style.display = 'inline';

    await fetch(`/api/containers/${containerId}/restart`, { method: 'POST' });

    // Poll the container status until it changes from "restarting"
    const checkStatus = async () => {
      const response = await fetch(`/api/containers/${containerId}`);
      const container = await response.json();
      if (container.status !== 'restarting') {
        statusText.textContent = container.status;
        loadingIcon.style.display = 'none';
        dataTable.row(row).invalidate().draw(false); // Refresh the specific row
      } else {
        setTimeout(checkStatus, 500); // Check again after 1 second
      }
    };

    checkStatus();
  } catch (error) {
    console.error('Error restarting container:', error);
  }
}

async function stopContainer(containerId) {
  try {
    const rows = dataTable.rows().nodes();
    let row;
    rows.each(function (r) {
      if (r.cells[7].textContent === containerId) {
        row = r;
      }
    });

    if (!row) {
      throw new Error('Container row not found');
    }

    const statusText = row.querySelector('.status-text');
    const loadingIcon = row.querySelector('.loading-icon');

    statusText.textContent = 'stopping';
    loadingIcon.style.display = 'inline';

    await fetch(`/api/containers/${containerId}/stop`, { method: 'POST' });

    statusText.textContent = 'stopped';
    loadingIcon.style.display = 'none';
    dataTable.row(row).invalidate().draw(false); // Refresh the specific row
  } catch (error) {
    console.error('Error stopping container:', error);
  }
}


async function restartContainer(containerId) {
  try {
    const rows = dataTable.rows().nodes();
    let row;
    rows.each(function (r) {
      if (r.cells[7].textContent === containerId) {
        row = r;
      }
    });

    if (!row) {
      throw new Error('Container row not found');
    }

    const statusText = row.querySelector('.status-text');
    const loadingIcon = row.querySelector('.loading-icon');

    statusText.textContent = 'restarting';
    loadingIcon.style.display = 'inline';

    await fetch(`/api/containers/${containerId}/restart`, { method: 'POST' });

    // Poll the container status until it changes from "restarting"
    const checkStatus = async () => {
      try {
        const response = await fetch(`/api/containers/${containerId}`);
        if (!response.ok) {
          throw new Error(`HTTP error! status: ${response.status}`);
        }
        const container = await response.json();
        if (container.status !== 'restarting') {
          statusText.textContent = container.status;
          loadingIcon.style.display = 'none';
          dataTable.row(row).invalidate().draw(false); // Refresh the specific row
        } else {
          setTimeout(checkStatus, 500); // Check again after 500 milliseconds
        }
      } catch (error) {
        console.error('Error fetching container status:', error);
      }
    };

    // Call the checkStatus function immediately
    checkStatus();
  } catch (error) {
    console.error('Error restarting container:', error);
  }
}

// Initialize components and fetch container data
document.addEventListener("DOMContentLoaded", function () {
  fetchContainers();
});